import { createServerRouter } from "typescript-rest-rpc/lib/server";
import Koa from "koa";
import KoaRouter from "koa-router";
import koaBody from "koa-body";
import { Backend, UserInfo } from "@shared/rpc/backend";
import { getRepository } from "typeorm";
import { KnownUser } from "@shared/db/entity/KnownUser";
import { client } from "./client";
import { GuildMember } from "discord.js";
import { isAuthorisedAsync } from "./util";

class BackendImpl implements Backend {
    async getModeratorUserInfo({ id }: { id: string }): Promise<UserInfo> {
        let repo = getRepository(KnownUser);
        
        let member: GuildMember;
        for(let [_, guild] of client.guilds) {
            try {
                member = await guild.fetchMember(id);
                break;
            } catch(e) {
                member = null;
            }
        }

        if(!member || !(await isAuthorisedAsync(member)))
            return undefined;

        return <UserInfo>{
            userId: member.user.id,
            username: member.user.tag,
            avatarURL: member.user.displayAvatarURL,
            guildId: member.guild.id,
            timestamp: new Date()
        };
    }
}

const PORT = 3010;
let app : Koa;
let router : KoaRouter;
export function startServer() {
    app = new Koa();
    app.use(koaBody({ multipart: true }));

    router = createServerRouter("/rpc", new BackendImpl());
    app.use(router.routes());
    app.use(router.allowedMethods());

    app.listen(PORT);
    console.log(`Started RPC service at ${PORT}`);
}